<?php
class LyricWiki {
	private $apiUrl = 'http://lyrics.wikia.com/api.php';
	private $synoinfo = '/etc/synoinfo.conf';
	private $getKeyValueBin = '/usr/syno/bin/synogetkeyvalue';

	public function __construct() {
	}

	public function getLyricsList($artist, $title, $info) {
		$count = 0;

		$searchUrl = sprintf(
			"%s?func=getSong&artist=%s&song=%s&fmt=realjson",
			$this->apiUrl, urlencode($artist), urlencode($title)
		);

		$content = $this->getContent($searchUrl);
		if (!$content) {
			return $count;
		}

		$obj = json_decode($content, TRUE);

		if ($obj['lyrics'] !== 'Not found') {
			$obj = $this->decodeUTF8($obj);
			$obj['lyrics'] = $this->addDemoHintString($obj['lyrics']);
			$id = $obj['lyrics'];
			$info->addTrackInfoToList($obj['artist'], $obj['song'], $id, $obj['lyrics']);

			$count = 1;
		}

		return $count;
	}

	public function getLyrics($id, $info) {
		$info->addLyrics($id, $id);

		return true;
	}

	private function getContent($url) {
		$curl = curl_init();

		curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 30);
		curl_setopt($curl, CURLOPT_TIMEOUT, 30);
		curl_setopt($curl, CURLOPT_FOLLOWLOCATION, TRUE);
		curl_setopt($curl, CURLOPT_VERBOSE, TRUE);
		curl_setopt($curl, CURLOPT_URL, $url);

		$result = curl_exec($curl);
		curl_close($curl);

		return $result;
	}

	private function getFirstMatch($string, $pattern) {
		if (1 === preg_match($pattern, $string, $matches)) {
			return $matches[1];
		}

		return FALSE;
	}

	private function decodeUTF8($obj) {
		$tryDecodeFunc = function($str) {
			if ($str) {
				$length = strlen($str);
				$temp = '';

				// if decode failed, remove character from end and decode again
				while (empty($temp) && $length > 0) {
					$temp = utf8_decode(substr($str, 0, $length));

					$jsonStr = json_encode($temp);
					if (empty($jsonStr) || 'null' === $jsonStr) {
						// will lead to json_encode() fail
						$temp = '';
					}
					$length -= 1;
				}

				$str = $temp;
			}

			return $str;
		};

		return array_map($tryDecodeFunc, $obj);
	}

	private function addDemoHintString($lyrics) {
		$demoHint = $this->getDemoHintString();

		if (!empty($demoHint)) {
			$extraLyrics = "\n\n\n$demoHint";
			return $lyrics.$extraLyrics;
		}
		return $lyrics;
	}

	private function getDemoHintString() {
		$language = $this->getDSMLanguage();
		$language = empty($language) ? 'enu' : $language;
		$file = "/var/packages/AudioStation/target/app/texts/$language/strings";
		$key = 'lyricwiki_only_for_demo';

		return shell_exec("$this->getKeyValueBin $file $key");
	}

	private function getDSMLanguage() {
		$key = 'language';
		$language = exec("$this->getKeyValueBin $this->synoinfo $key");

		if ('def' == $language) {
			$language = $this->getBrowserLanguage();
			$language = empty($language) ? $this->getDSMCodePage() : $language;
		}
		return $language;
	}

	private function getDSMCodePage() {
		$key = 'codepage';

		return exec("$this->getKeyValueBin $this->synoinfo $key");
	}

	private function getBrowserLanguage() {
		$file = '/etc.defaults/synoinfo.conf';
		$key = 'supplang';
		$supportLang = exec("$this->getKeyValueBin $file $key");

		$arr = preg_split( "/,|;/", $_SERVER['HTTP_ACCEPT_LANGUAGE']);
		$length = count($arr);
		for ($i = 0; $i < $length; $i++) {
			if (!strncmp($arr[$i], 'zh-Hant', 7) ||
				!strcasecmp($arr[$i], 'zh-tw') ||
				!strcasecmp($arr[$i], 'zh-hk') ||
				!strcasecmp($arr[$i], 'zh-mo')) {
				if (!strstr($supportLang, 'cht')) {
					continue;
				}
				$language = 'cht';
			} else if (!strncasecmp($arr[$i], 'zh', 2)) {
				if (!strstr($supportLang, 'chs')) {
					continue;
				}
				$language = 'chs';
			} else if (!strcasecmp($arr[$i], 'en') ||
						!strncasecmp($arr[$i], 'en-', 3)) {
				if (!strstr($supportLang, 'enu')) {
					continue;
				}
				$language = 'enu';
			} else if (!strcasecmp($arr[$i], 'de') ||
						!strncasecmp($arr[$i], 'de-', 3)) {
				if (!strstr($supportLang, 'ger')) {
					continue;
				}
				$language = 'ger';
			} else if (!strcasecmp($arr[$i], 'ja') ||
						!strncasecmp($arr[$i], 'ja-', 3)) {
				if (!strstr($supportLang, 'jpn')) {
					continue;
				}
				$language = 'jpn';
			} else if (!strcasecmp($arr[$i], 'ko') ||
						!strncasecmp($arr[$i], 'ko-', 3)) {
				if (!strstr($supportLang, 'krn')) {
					continue;
				}
				$language = 'krn';
			} else if (!strcasecmp($arr[$i], 'es') ||
						!strncasecmp($arr[$i], 'es-', 3)) {
				if (!strstr($supportLang, 'spn')) {
					continue;
				}
				$language = 'spn';
			} else if (!strcasecmp($arr[$i], 'fr') ||
						!strncasecmp($arr[$i], 'fr-', 3)) {
				if (!strstr($supportLang, 'fre')) {
					continue;
				}
				$language = 'fre';
			} else if (!strcasecmp($arr[$i], 'it') ||
						!strncasecmp($arr[$i], 'it-', 3)) {
				if (!strstr($supportLang, 'ita')) {
					continue;
				}
				$language = 'ita';
			} else if (!strncasecmp($arr[$i], 'pt-br', 5)) {
				if (!strstr($supportLang, 'ptb')) {
					continue;
				}
				$language = 'ptb';
			} else if (!strcasecmp($arr[$i], 'pt') ||
						!strncasecmp($arr[$i], 'pt-', 3)) {
				if (!strstr($supportLang, 'ptg')) {
					continue;
				}
				$language = 'ptg';
			} else if (!strcasecmp($arr[$i], 'hu') ||
						!strncasecmp($arr[$i], 'hu-', 3)) {
				if (!strstr($supportLang, 'hun')) {
					continue;
				}
				$language = 'hun';
			} else if (!strcasecmp($arr[$i], 'tr') ||
						!strncasecmp($arr[$i], 'tr-', 3)) {
				if (!strstr($supportLang, 'trk')) {
					continue;
				}
				$language = 'trk';
			} else if (!strcasecmp($arr[$i], 'ru') ||
						!strncasecmp($arr[$i], 'ru-', 3)) {
				if (!strstr($supportLang, 'rus')) {
					continue;
				}
				$language = 'rus';
			} else if (!strcasecmp($arr[$i], 'da') ||
						!strncasecmp($arr[$i], 'da-', 3)) {
				if (!strstr($supportLang, 'dan')) {
					continue;
				}
				$language = 'dan';
			} else if (!strcasecmp($arr[$i], 'sv') ||
						!strncasecmp($arr[$i], 'sv-', 3)) {
				if (!strstr($supportLang, 'sve')) {
					continue;
				}
				$language = 'sve';
			} else if (!strcasecmp($arr[$i], 'no') ||
						!strncasecmp($arr[$i], 'no-', 3) ||
						!strcasecmp($arr[$i], '"nb') ||
						!strncasecmp($arr[$i], 'nb-', 3) ||
						!strcasecmp($arr[$i], 'nn') ||
						!strncasecmp($arr[$i], 'nn-', 3)) {
				if (!strstr($supportLang, 'nor')) {
					continue;
				}
				$language = 'nor';
			} else if (!strcasecmp($arr[$i], 'nl') ||
						!strncasecmp($arr[$i], 'nl-', 3)) {
				if (!strstr($supportLang, 'nld')) {
					continue;
				}
				$language = 'nld';
			} else if (!strcasecmp($arr[$i], 'pl') ||
						!strncasecmp($arr[$i], 'pl-', 3)) {
				if (!strstr($supportLang, 'plk')) {
					continue;
				}
				$language = 'plk';
			} else if (!strcasecmp($arr[$i], 'cs') ||
						!strncasecmp($arr[$i], 'cs-', 3)) {
				if (!strstr($supportLang, 'csy')) {
					continue;
				}
				$language = 'csy';
			} else {
				continue;
			}
			break;
		}

		return $language;
	}
}
?>

